# Birdie | Task 2
# Author: Simon Stueckemann
# Date: 2/2/2015
# Version 0.1
#
# Birdie is a simple game that lets the player fly a bird through the clouds.

import sys, pygame
from pygame.locals import *
import random

ANIMATE_BIRD_E = USEREVENT + 1

# The following represents a bird on the screen. 
# You can create a new bird by typing "Bird()". You do not have to understand 
# the following program code. We will discuss some of it more in the coming weeks.
class Bird(pygame.sprite.Sprite):
    def __init__(self):
        super(Bird, self).__init__()
        self.images = []
        self.images.append(pygame.image.load('flappingbird-0.tiff'))
        self.images.append(pygame.image.load('flappingbird-1.tiff'))
        self.images.append(pygame.image.load('flappingbird-2.tiff'))
        self.images.append(pygame.image.load('flappingbird-3.tiff'))

        self.index = 0
        self.image = self.images[self.index]

        self.rect = pygame.Rect(100, 130, 95, 108)

    def move_up(self):
        if self.rect.y - 5 >= 0:
            self.rect.y -= 5

    def move_down(self):
        if self.rect.y + 5 + self.rect.height <= 424:
            self.rect.y += 5

    def move_right(self):
        if self.rect.x + self.rect.width <= 624:
            self.rect.x += 5

    def move_left(self):
        if self.rect.x - 5 >= 0:
            self.rect.x -= 5

    def flap(self):
        self.index += 1
        if self.index >= len(self.images):
            self.index = 0
        self.image = self.images[self.index]

# The following represents an obstacle that Birdie has to avoid.
# You can create a new obstacle by calling Obstacle(is_top,stop_y) for some appropriate values is_top and stop_y
# The obstacle reaches from either the top to some place in the middle or from the bottom to the middle. 
# is_top is either True or False and if True, the new obstacle will reach from the top down and will be 
# stop_y points long. 
class Obstacle(pygame.sprite.Sprite):
    def __init__(self, is_top, stop_y):
        super(Obstacle, self).__init__()
        if is_top:
            h = stop_y
        else:
            h = height - stop_y

        self.image = pygame.Surface([30, h])
        self.image.fill(Color("brown"))

        top = 0 if is_top else stop_y

        self.rect = pygame.Rect(width - 50, top, 30, h)

    def is_visible(self):
        return self.rect.colliderect(screen.get_rect())

    def update(self):
        self.rect.x -= 5


# Pygame helps us set up our program for game development
# This line makes sure the computer knows that we want to make a game
# In technical terms, this "initialises the pygame library"
pygame.init()

# The very first step is to create a window where we can draw the 
# bird and the cloud
width = 624
height = 424
# Instead of keeping width and height separately, we list them together as a "pair": (624, 424)
size = (width, height)

# Our game is called "Birdie" and thus the window should be named accordingly
pygame.display.set_caption("Birdie")

# This line finally opens up a window in which we can draw. Notice that we pass 
# the size pair from before so that the computer knows how large the window 
# should be
screen = pygame.display.set_mode(size)

# The next step is to load the image of the cloud that we use as a background.
cloud = pygame.image.load("cloud.jpg")

# Next we load Birdie and add him/her to the group of things 
# we want to draw at the same time
bird = Bird()
sprite_group = pygame.sprite.Group(bird)

obstacles = pygame.sprite.Group()

clock = pygame.time.Clock()
pygame.time.set_timer(ANIMATE_BIRD_E, 100)

pygame.key.set_repeat(1, 30)

# This function adds a new obstacle to the right side of the screen with a window at a random height
def add_obstacle():
    top = random.randrange(20, 250)
    top_obstacle = Obstacle(True, top)
    bottom_obstacle = Obstacle(False, top + 150)

    top_obstacle.add(obstacles)
    bottom_obstacle.add(obstacles)

# Here we handle all of the special events that might come in
# For example, if a user wants to exit the game this is done here
def handle_events(events):
    for event in events:
        if event.type == pygame.QUIT: 
            sys.exit()
        elif event.type == ANIMATE_BIRD_E:
            bird.flap()

# We need to manually make sure that the bird actually moves 
# when we press the arrow keys
def handle_movement():
    # Here we get all the keys that are currently pressed
    keys = pygame.key.get_pressed()

    if keys[K_DOWN]:
        bird.move_down()

    if keys[K_UP]:
        bird.move_up()

    if keys[K_RIGHT]:
        bird.move_right()

    if keys[K_LEFT]:
        bird.move_left()


add_obstacle()

# This is the main game event loop which contains the program code 
# that is repeated until the player wants to end the game.
while True:
    clock.tick(30)

    # We first deal with all the special events that could happen, e.g. 
    # the user wanting to exit the game
    handle_events(pygame.event.get())

    # Now we need to make sure the user's arrow key presses actually 
    # move the bird
    handle_movement()

    # Now we start drawing to the screen:
    # We first draw the background cloud, on top of which we then place the bird
    screen.blit(cloud, cloud.get_rect())

    obstacles.draw(screen)
    sprite_group.draw(screen)

    # Finally, after everything is drawn, we tell the computer that we are 
    # ready to display the new changes to the user
    pygame.display.flip()
